<?php
/*
Plugin Name: Update Unique Keys
Plugin URI: http://passavanti.name/wordpress-update-unique-keys
Description: This plugin will automatically set and/or update the Authenication Unique Keys in the wp-config.php file.
Version: 1.0.2
Author: Brian Passavanti
Author URI: http://passavanti.name

Copyright 2010  BRIAN_PASSAVANTI  (email : gottaloveit@gmail.com)

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
class updateuniquekeys
{ 
	private $prefix;
	private $name;
	private $dir;

	private $filename;
	private $keys = array();
	private $isWritable = false;
	private $lastUpdateLine = 0;
	private $originalFileArray = array();
	private $lastLineOriginalFileArray = 0;
	private $updateMap = array();
	private $addLines = array();    

	public function __construct($prefix, $name, $dir) {
		$this->prefix = $prefix;
		$this->name = $name;
		$this->dir = $dir;
		$this->filename = ABSPATH . DIRECTORY_SEPARATOR . 'wp-config.php';
		$this->isWritable = is_writable($this->filename);
		$this->keys = file('http://api.wordpress.org/secret-key/1.1/salt');
	}

	private function findLine($str,$key) {
		$exists = false;
		foreach ($this->originalFileArray as $k=>$v) {
			$pos = strpos($v, $str);
			if ($pos !== false) {
				$exists = true;
				$this->updateMap[$key] = $k;
				$this->lastUpdateLine = $k;
			}
		}
		if (!$exists) {
			$this->addLines[] = $key;
		}
	}

	private function moveKeys() {
		for ($a = $this->lastLineOriginalFileArray;$a>$this->lastUpdateLine;$a--) {
			$this->originalFileArray[$a+count($this->addLines)] = $this->originalFileArray[$a]; 
		}
	}
    
	private function addNewLines() {
		$b = $this->lastUpdateLine + 1;
		for ($a = 0;$a<count($this->addLines);$a++) {
			$this->originalFileArray[$b] = $this->keys[$this->addLines[$a]]; 
			$b++;
		}
	}

	private function updateLines() {
		foreach ($this->updateMap as $k=>$v) {
			$this->originalFileArray[$v] = $this->keys[$k];
		}
	}

	public function update() {
		$this->originalFileArray = file($this->filename);
		$this->lastLineOriginalFileArray = max(array_keys($this->originalFileArray));

		foreach ($this->keys as $k=>$v) {
			$pos = strpos($v, ",");
			$str = substr($v,0,$pos);
			$this->findLine($str,$k);
		}

		$this->moveKeys();
		$this->addNewLines();
		$this->updateLines();
		ksort($this->originalFileArray);
		file_put_contents($this->filename,implode($this->originalFileArray));
		return true;
	}

	public function ActionAdminMenu() {
		add_options_page('Update Unique Keys', 'Update Unique Keys', 10, 'updateuniquekeys', array($this, 'GetConfigHTML'));
	}

	public function GetConfigHTML() {

		$hidden_field_name = 'uuk_submit_hidden';

		if ((array_key_exists($hidden_field_name,$_POST)) && ($_POST[ $hidden_field_name ] == 'uuksubmit' )) {
			if (!$this->isWritable) {
				echo '<br /><br /><br /><div class="updated"><p><strong>Config File Not Writable.</strong></p></div>';
				echo '<div class="updated"><p>Manually edit the file wp-config.php in your blog root with the following:</p>';
				echo '<pre>';
				echo htmlspecialchars(implode($this->keys));
				echo '</pre></div>';
			} else {
				if ($this->update()) {
					echo '<div class="updated"><p><strong>Config File Updated!</strong></p></div>';
					echo '<div class="updated">You have automatically been logged out, due to the new keys.  Your password is the same.</div>';
				} else {
					echo '<div class="updated"><p><strong>some sort of error</strong></p></div>';
				}
			}
		}

		$output =  '
				<div class="wrap">
					<form method="post" action="">
						<h2>Update Unique Keys</h2>
						<table class="form-table">
							<tbody>
								<tr valign="top">
									<td>
										Press the button below to generate the keys/salts and update the wp-config.php file.
									</td>
								</tr>
							</tbody>
						</table>
						<input type="hidden" name="'.$hidden_field_name.'" value="uuksubmit" />
						<p class="submit">
							<input type="submit" name="Submit" value="Update" />
						</p>
					</form>
					<br /><br />
					Donations appreciated, thank you!<br />
					<form action="https://www.paypal.com/cgi-bin/webscr" method="post">
					<input type="hidden" name="cmd" value="_s-xclick">
					<input type="hidden" name="hosted_button_id" value="2HAFRNX23EE88">
					<input type="image" src="https://www.paypal.com/en_US/i/btn/btn_donateCC_LG.gif" border="0" name="submit" alt="PayPal - The safer, easier way to pay online!">
					<img alt="" border="0" src="https://www.paypal.com/en_US/i/scr/pixel.gif" width="1" height="1">
					</form>
				</div>
			';

		echo $output;
	}

}
?>